@@ -3,33 +3,38 @@
 # This work is licensed under the GNU GPLv2 or later.
 # See the COPYING file in the top-level directory.
 
-import logging
 import os
 import re
 import sys
-import traceback
-import unittest
+
+import pytest
 
 from tests import utils
 
+import virtinst.progress
 from virtinst import Installer
 from virtinst import Guest
-from virtinst import util
+from virtinst import log
+
+# These are all functional tests
+os.environ.pop("VIRTINST_TEST_SUITE", None)
 
 
 class _URLTestData(object):
     """
     Class that tracks all data needed for a single URL test case.
-    Data is stored in test_urls.ini
+    Data is stored in data/test_urls.ini
     """
     def __init__(self, name, url, detectdistro,
-            testxen, testshortcircuit, kernelarg, kernelregex):
+            testxen, testshortcircuit, kernelarg, kernelregex,
+            skip_libosinfo):
         self.name = name
         self.url = url
         self.detectdistro = detectdistro
         self.arch = self._find_arch()
         self.kernelarg = kernelarg
         self.kernelregex = kernelregex
+        self.skip_libosinfo = skip_libosinfo
 
         self.testxen = testxen
 
@@ -62,7 +67,12 @@ hvmguest.os.os_type = "hvm"
 xenguest = Guest(testconn)
 xenguest.os.os_type = "xen"
 
-meter = util.make_meter(quiet=not utils.clistate.debug)
+meter = virtinst.progress.make_meter(quiet=not utils.TESTCONFIG.debug)
+
+if utils.TESTCONFIG.url_skip_libosinfo:
+    os.environ["VIRTINST_TEST_SUITE_FORCE_LIBOSINFO"] = "0"
+elif utils.TESTCONFIG.url_force_libosinfo:
+    os.environ["VIRTINST_TEST_SUITE_FORCE_LIBOSINFO"] = "1"
 
 
 def _sanitize_osdict_name(detectdistro):
@@ -71,6 +81,37 @@ def _sanitize_osdict_name(detectdistro):
     return detectdistro
 
 
+def _skipmsg(testdata):
+    is_iso = testdata.url.lower().endswith(".iso")
+    distname = testdata.name
+
+    if utils.TESTCONFIG.url_iso_only and not is_iso:
+        return "skipping non-iso test"
+    elif utils.TESTCONFIG.url_only and is_iso:
+        return "skipping non-url test"
+
+    if not utils.TESTCONFIG.url_force_libosinfo:
+        return
+    if testdata.skip_libosinfo:
+        return "force-libosinfo requested but test has skip_libosinfo set"
+    if is_iso:
+        return
+
+    # If --force-libosinfo used, don't run tests that we know libosinfo
+    # can't detect, non-treeinfo URLs basically
+    if ("ubuntu" in distname or
+        "debian" in distname or
+        "mageia" in distname or
+        "opensuse10" in distname or
+        "opensuse11" in distname or
+        "opensuse12" in distname or
+        "opensuse13" in distname or
+        "opensuseleap-42" in distname or
+        "generic" in distname or
+        testdata.url.startswith("ftp:/")):
+        return "skipping known busted libosinfo URL tests"
+
+
 def _testGuest(testdata, guest):
     distname = testdata.name
     arch = testdata.arch
@@ -78,17 +119,22 @@ def _testGuest(testdata, guest):
     checkdistro = testdata.detectdistro
 
     guest.os.arch = arch
+    guest.set_os_name("generic")
     if testdata.testshortcircuit:
         guest.set_os_name(checkdistro)
 
+    msg = _skipmsg(testdata)
+    if msg:
+        raise pytest.skip(msg)
+
     installer = Installer(guest.conn, location=url)
     try:
         detected_distro = installer.detect_distro(guest)
-    except Exception:
-        raise AssertionError("\nFailed in installer detect_distro():\n"
+    except Exception as e:
+        msg = ("\nFailed in installer detect_distro():\n"
             "name   = %s\n"
-            "url    = %s\n\n%s" %
-            (distname, url, "".join(traceback.format_exc())))
+            "url    = %s\n\n%s" % (distname, url, str(e)))
+        raise type(e)(msg).with_traceback(sys.exc_info()[2]) from None
 
     # Make sure the stores are reporting correct distro name/variant
     if checkdistro != detected_distro:
@@ -103,17 +149,18 @@ def _testGuest(testdata, guest):
     if guest is xenguest:
         return
 
+
     # Do this only after the distro detection, since we actually need
     # to fetch files for that part
     treemedia = installer._treemedia  # pylint: disable=protected-access
     fetcher = treemedia._cached_fetcher  # pylint: disable=protected-access
-    def fakeAcquireFile(filename):
-        logging.debug("Fake acquiring %s", filename)
+    def fakeAcquireFile(filename, fullurl=None):
+        log.debug("Fake acquiring filename=%s fullurl=%s", filename, fullurl)
         return filename
     fetcher.acquireFile = fakeAcquireFile
 
     # Fetch regular kernel
-    kernel, initrd, kernelargs = treemedia.prepare(guest, meter)
+    kernel, initrd, kernelargs = treemedia.prepare(guest, meter, None)
     dummy = initrd
     if testdata.kernelregex and not re.match(testdata.kernelregex, kernel):
         raise AssertionError("kernel=%s but testdata.kernelregex='%s'" %
@@ -133,50 +180,36 @@ def _testURL(testdata):
     """
     Test that our URL detection logic works for grabbing kernels
     """
+    sys.stdout.write("\nTesting %-25s " % testdata.name)
+    sys.stdout.flush()
+
     testdata.detectdistro = _sanitize_osdict_name(testdata.detectdistro)
     _testGuest(testdata, hvmguest)
     if testdata.testxen:
         _testGuest(testdata, xenguest)
 
 
-def _testURLWrapper(testdata):
-    os.environ.pop("VIRTINST_TEST_SUITE", None)
+def test001BadURL():
+    badurl = "http://aksdkakskdfa-idontexist.com/foo/tree"
 
-    sys.stdout.write("\nTesting %-25s " % testdata.name)
-    sys.stdout.flush()
+    with pytest.raises(ValueError, match=".*maybe you mistyped.*"):
+        installer = Installer(hvmguest.conn, location=badurl)
+        installer.detect_distro(hvmguest)
 
-    return _testURL(testdata)
+    # Non-existent cdrom fails
+    with pytest.raises(ValueError, match=".*non-existent path.*"):
+        installer = Installer(hvmguest.conn, cdrom="/not/exist/foobar")
+        assert installer.detect_distro(hvmguest) is None
 
-
-# Register tests to be picked up by unittest
-class URLTests(unittest.TestCase):
-    def test001BadURL(self):
-        badurl = "http://aksdkakskdfa-idontexist.com/foo/tree"
-
-        try:
-            installer = Installer(hvmguest.conn, location=badurl)
-            installer.detect_distro(hvmguest)
-            raise AssertionError("Expected URL failure")
-        except ValueError as e:
-            self.assertTrue("maybe you mistyped" in str(e))
-
-        # Non-existent cdrom fails
-        try:
-            installer = Installer(hvmguest.conn, cdrom="/i/dont/exist/foobar")
-            self.assertEqual(None, installer.detect_distro(hvmguest))
-            raise AssertionError("Expected cdrom failure")
-        except ValueError as e:
-            self.assertTrue("non-existent path" in str(e))
-
-        # Ensure existing but non-distro file doesn't error
-        installer = Installer(hvmguest.conn, cdrom="/dev/null")
-        self.assertEqual(None, installer.detect_distro(hvmguest))
+    # Ensure existing but non-distro file doesn't error
+    installer = Installer(hvmguest.conn, cdrom="/dev/null")
+    assert installer.detect_distro(hvmguest) is None
 
 
 def _make_tests():
     import configparser
     cfg = configparser.ConfigParser()
-    cfg.read("tests/test_urls.ini")
+    cfg.read("tests/data/test_urls.ini")
 
     manualpath = "~/.config/virt-manager/test_urls_manual.ini"
     cfg.read(os.path.expanduser(manualpath))
@@ -186,18 +219,23 @@ def _make_tests():
     urls = {}
     for name in cfg.sections():
         vals = dict(cfg.items(name))
-        d = _URLTestData(name, vals["url"],
-                vals.get("distro", None),
+        url = vals["url"]
+
+        if "distro" not in vals:
+            print("url needs an explicit distro= value: %s" % url)
+            sys.exit(1)
+        d = _URLTestData(name, url, vals["distro"],
                 vals.get("testxen", "0") == "1",
                 vals.get("testshortcircuit", "0") == "1",
                 vals.get("kernelarg", None),
-                vals.get("kernelregex", None))
+                vals.get("kernelregex", None),
+                vals.get("skiplibosinfo", "0") == "1")
         urls[d.name] = d
 
     for key, testdata in sorted(urls.items()):
         def _make_wrapper(d):
-            return lambda _self: _testURLWrapper(d)
-        setattr(URLTests, "testURL%s" % key.replace("-", "_"),
-                _make_wrapper(testdata))
+            return lambda: _testURL(d)
+        methodname = "test_URL%s" % key.replace("-", "_")
+        globals()[methodname] = _make_wrapper(testdata)
 
 _make_tests()
