@@ -1,47 +1,72 @@
-#
 # Copyright (C) 2006, 2012-2015 Red Hat, Inc.
 # Copyright (C) 2006 Daniel P. Berrange <berrange@redhat.com>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
-#
-import os
-import logging
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import Gtk
 
-from virtinst import CPU
-from .keyring import vmmKeyring, vmmSecret
+from virtinst import DomainCpu
+from virtinst import log
 
-running_config = None
+from .lib.inspection import vmmInspection
 
 
-class SettingsWrapper(object):
-    def __init__(self, settings_id):
+CSSDATA = """
+/* Lighter colored text in some wizard summary fields */
+.vmm-lighter {
+    color: @insensitive_fg_color
+}
+
+/* Text on the blue header in our wizards */
+.vmm-header-text {
+    color: white
+}
+
+/* Subtext on the blue header in our wizards */
+.vmm-header-subtext {
+    color: #59B0E2
+}
+
+/* The blue header */
+.vmm-header {
+    background-color: #0072A8
+}
+"""
+
+
+class _SettingsWrapper(object):
+    """
+    Wrapper class to simplify interacting with gsettings APIs.
+    Basically it allows simple get/set of gconf style paths, and
+    we internally convert it to the settings nested hierarchy. Makes
+    client code much smaller.
+    """
+    def __init__(self, settings_id, gsettings_keyfile):
         self._root = settings_id
-        self._settings = Gio.Settings.new(self._root)
+
+        if gsettings_keyfile:
+            backend = Gio.keyfile_settings_backend_new(gsettings_keyfile, "/")
+        else:
+            backend = Gio.SettingsBackend.get_default()
+
+        self._settings = Gio.Settings.new_with_backend(self._root, backend)
 
         self._settingsmap = {"": self._settings}
         self._handler_map = {}
+
         for child in self._settings.list_children():
             childschema = self._root + "." + child
-            self._settingsmap[child] = Gio.Settings.new(childschema)
+            self._settingsmap[child] = Gio.Settings.new_with_backend(
+                    childschema, backend)
 
 
+    ###################
+    # Private helpers #
+    ###################
+
     def _parse_key(self, key):
         value = key.strip("/")
         settingskey = ""
@@ -49,6 +74,15 @@ class SettingsWrapper(object):
             settingskey, value = value.rsplit("/", 1)
         return settingskey, value
 
+    def _find_settings(self, key):
+        settingskey, value = self._parse_key(key)
+        return self._settingsmap[settingskey], value
+
+
+    ###############
+    # Public APIs #
+    ###############
+
     def make_vm_settings(self, key):
         """
         Initialize per-VM relocatable schema if necessary
@@ -77,14 +111,6 @@ class SettingsWrapper(object):
                 schema, path)
         return True
 
-    def _find_settings(self, key):
-        settingskey, value = self._parse_key(key)
-        return self._settingsmap[settingskey], value
-
-    def _cmd_helper(self, cmd, key, *args, **kwargs):
-        settings, key = self._find_settings(key)
-        return getattr(settings, cmd)(key, *args, **kwargs)
-
     def notify_add(self, key, cb, *args, **kwargs):
         settings, key = self._find_settings(key)
         def wrapcb(*ignore):
@@ -97,12 +123,13 @@ class SettingsWrapper(object):
         return settings.disconnect(h)
 
     def get(self, key):
-        return self._cmd_helper("get_value", key).unpack()
+        settings, key = self._find_settings(key)
+        return settings.get_value(key).unpack()
     def set(self, key, value, *args, **kwargs):
-        fmt = self._cmd_helper("get_value", key).get_type_string()
-        return self._cmd_helper("set_value", key,
-                                GLib.Variant(fmt, value),
-                                *args, **kwargs)
+        settings, key = self._find_settings(key)
+        fmt = settings.get_value(key).get_type_string()
+        return settings.set_value(key, GLib.Variant(fmt, value),
+                                  *args, **kwargs)
 
 
 class vmmConfig(object):
@@ -116,31 +143,38 @@ class vmmConfig(object):
     # Metadata mapping for browse types. Prob shouldn't go here, but works
     # for now.
     browse_reason_data = {
-        CONFIG_DIR_IMAGE : {
-            "enable_create" : True,
-            "storage_title" : _("Locate or create storage volume"),
-            "local_title"   : _("Locate existing storage"),
-            "dialog_type"   : Gtk.FileChooserAction.SAVE,
-            "choose_button" : Gtk.STOCK_OPEN,
+        CONFIG_DIR_IMAGE: {
+            "enable_create":  True,
+            "storage_title":  _("Locate or create storage volume"),
+            "local_title":    _("Locate existing storage"),
+            "dialog_type":    Gtk.FileChooserAction.SAVE,
+            "choose_button":  Gtk.STOCK_OPEN,
+            "gsettings_key": "image",
         },
 
-        CONFIG_DIR_ISO_MEDIA : {
-            "enable_create" : False,
-            "storage_title" : _("Locate ISO media volume"),
-            "local_title"   : _("Locate ISO media"),
+        CONFIG_DIR_SCREENSHOT: {
+            "gsettings_key": "screenshot",
         },
 
-        CONFIG_DIR_FLOPPY_MEDIA : {
-            "enable_create" : False,
-            "storage_title" : _("Locate floppy media volume"),
-            "local_title"   : _("Locate floppy media"),
+        CONFIG_DIR_ISO_MEDIA: {
+            "enable_create":  False,
+            "storage_title":  _("Locate ISO media volume"),
+            "local_title":    _("Locate ISO media"),
+            "gsettings_key": "media",
         },
 
-        CONFIG_DIR_FS : {
-            "enable_create" : False,
-            "storage_title" : _("Locate directory volume"),
-            "local_title"   : _("Locate directory volume"),
-            "dialog_type"   : Gtk.FileChooserAction.SELECT_FOLDER,
+        CONFIG_DIR_FLOPPY_MEDIA: {
+            "enable_create":  False,
+            "storage_title":  _("Locate floppy media volume"),
+            "local_title":    _("Locate floppy media"),
+            "gsettings_key": "media",
+        },
+
+        CONFIG_DIR_FS: {
+            "enable_create":  False,
+            "storage_title":  _("Locate directory volume"),
+            "local_title":    _("Locate directory volume"),
+            "dialog_type":    Gtk.FileChooserAction.SELECT_FOLDER,
         },
     }
 
@@ -148,52 +182,67 @@ class vmmConfig(object):
     CONSOLE_SCALE_FULLSCREEN = 1
     CONSOLE_SCALE_ALWAYS = 2
 
-    def __init__(self, appname, CLIConfig, test_first_run=False):
-        self.appname = appname
-        self.appversion = CLIConfig.version
-        self.conf_dir = "/org/virt-manager/%s/" % self.appname
-        self.ui_dir = CLIConfig.ui_dir
-        self.test_first_run = bool(test_first_run)
+    _instance = None
 
-        self.conf = SettingsWrapper("org.virt-manager.virt-manager")
+    @classmethod
+    def get_instance(cls, *args, **kwargs):
+        if not cls._instance:
+            cls._instance = vmmConfig(*args, **kwargs)
+        return cls._instance
+
+    @classmethod
+    def is_initialized(cls):
+        return bool(cls._instance)
+
+    def __init__(self, BuildConfig, CLITestOptions):
+        self.appname = "virt-manager"
+        self.appversion = BuildConfig.version
+        self.conf_dir = "/org/virt-manager/%s/" % self.appname
+        self.ui_dir = BuildConfig.ui_dir
+
+        self.conf = _SettingsWrapper("org.virt-manager.virt-manager",
+                CLITestOptions.gsettings_keyfile)
+
+        self.CLITestOptions = CLITestOptions
+        if self.CLITestOptions.xmleditor_enabled:
+            self.set_xmleditor_enabled(True)
+        if self.CLITestOptions.enable_libguestfs:
+            self.set_libguestfs_inspect_vms(True)
+        if self.CLITestOptions.disable_libguestfs:
+            self.set_libguestfs_inspect_vms(False)
 
         # We don't create it straight away, since we don't want
         # to block the app pending user authorization to access
         # the keyring
-        self.keyring = None
+        self._keyring = None
 
-        self.default_qemu_user = CLIConfig.default_qemu_user
-        self.preferred_distros = CLIConfig.preferred_distros
-        self.hv_packages = CLIConfig.hv_packages
-        self.libvirt_packages = CLIConfig.libvirt_packages
-        self.askpass_package = CLIConfig.askpass_package
-        self.default_graphics_from_config = CLIConfig.default_graphics
-        self.default_hvs = CLIConfig.default_hvs
-        self.cli_usbredir = None
+        self.default_graphics_from_config = BuildConfig.default_graphics
+        self.default_hvs = BuildConfig.default_hvs
 
         self.default_storage_format_from_config = "qcow2"
-        self.cpu_default_from_config = "host-cpu-model"
         self.default_console_resizeguest = 0
-        self.default_add_spice_usbredir = "yes"
 
         self._objects = []
+        self.color_insensitive = None
+        self._init_css()
 
-        self.support_inspection = self.check_inspection()
+    def _init_css(self):
+        from gi.repository import Gdk
+        screen = Gdk.Screen.get_default()
 
-        self._spice_error = None
+        css_provider = Gtk.CssProvider()
+        css_provider.load_from_data(CSSDATA.encode("utf-8"))
 
-        global running_config
-        running_config = self
+        context = Gtk.StyleContext()
+        context.add_provider_for_screen(screen, css_provider,
+             Gtk.STYLE_PROVIDER_PRIORITY_USER)
 
+        found, color = context.lookup_color("insensitive_fg_color")
+        if not found:  # pragma: no cover
+            log.debug("Didn't find insensitive_fg_color in theme")
+            return
+        self.color_insensitive = color.to_string()
 
-    def check_inspection(self):
-        try:
-            # Check we can open the Python guestfs module.
-            from guestfs import GuestFS  # pylint: disable=import-error
-            g = GuestFS(close_on_exit=False)
-            return bool(getattr(g, "add_libvirt_dom", None))
-        except:
-            return False
 
     # General app wide helpers (gsettings agnostic)
 
@@ -208,6 +257,11 @@ class vmmConfig(object):
         ret = ["vnc", "spice"]
         return ret
 
+    def inspection_supported(self):
+        if not vmmInspection.libguestfs_installed():
+            return False  # pragma: no cover
+        return self.get_libguestfs_inspect_vms()
+
     def remove_notifier(self, h):
         self.conf.notify_remove(h)
 
@@ -321,14 +375,6 @@ class vmmConfig(object):
     def on_keys_combination_changed(self, cb):
         return self.conf.notify_add("/console/grab-keys", cb)
 
-    # This key is not intended to be exposed in the UI yet
-    def get_keyboard_grab_default(self):
-        return self.conf.get("/console/grab-keyboard")
-    def set_keyboard_grab_default(self, val):
-        self.conf.set("/console/grab-keyboard", val)
-    def on_keyboard_grab_default_changed(self, cb):
-        return self.conf.notify_add("/console/grab-keyboard", cb)
-
     # Confirmation preferences
     def get_confirm_forcepoweroff(self):
         return self.conf.get("/confirm/forcepoweroff")
@@ -338,14 +384,11 @@ class vmmConfig(object):
         return self.conf.get("/confirm/pause")
     def get_confirm_removedev(self):
         return self.conf.get("/confirm/removedev")
-    def get_confirm_interface(self):
-        return self.conf.get("/confirm/interface-power")
     def get_confirm_unapplied(self):
         return self.conf.get("/confirm/unapplied-dev")
     def get_confirm_delstorage(self):
         return self.conf.get("/confirm/delete-storage")
 
-
     def set_confirm_forcepoweroff(self, val):
         self.conf.set("/confirm/forcepoweroff", val)
     def set_confirm_poweroff(self, val):
@@ -354,8 +397,6 @@ class vmmConfig(object):
         self.conf.set("/confirm/pause", val)
     def set_confirm_removedev(self, val):
         self.conf.set("/confirm/removedev", val)
-    def set_confirm_interface(self, val):
-        self.conf.set("/confirm/interface-power", val)
     def set_confirm_unapplied(self, val):
         self.conf.set("/confirm/unapplied-dev", val)
     def set_confirm_delstorage(self, val):
@@ -371,14 +412,30 @@ class vmmConfig(object):
         self.conf.set("/system-tray", val)
 
 
+    # XML editor enabled
+    def on_xmleditor_enabled_changed(self, cb):
+        return self.conf.notify_add("/xmleditor-enabled", cb)
+    def get_xmleditor_enabled(self):
+        return self.conf.get("/xmleditor-enabled")
+    def set_xmleditor_enabled(self, val):
+        self.conf.set("/xmleditor-enabled", val)
+
+
+    # Libguestfs VM inspection
+    def get_libguestfs_inspect_vms(self):
+        return self.conf.get("/enable-libguestfs-vm-inspection")
+    def set_libguestfs_inspect_vms(self, val):
+        self.conf.set("/enable-libguestfs-vm-inspection", val)
+
+
     # Stats history and interval length
     def get_stats_history_length(self):
         return 120
     def get_stats_update_interval(self):
+        if self.CLITestOptions.short_poll:
+            return .1
         interval = self.conf.get("/stats/update-interval")
-        if interval < 1:
-            return 1
-        return interval
+        return max(interval, 1)
     def set_stats_update_interval(self, interval):
         self.conf.set("/stats/update-interval", interval)
     def on_stats_update_interval_changed(self, cb):
@@ -413,26 +470,11 @@ class vmmConfig(object):
     def on_stats_enable_memory_poll_changed(self, cb, row=None):
         return self.conf.notify_add("/stats/enable-memory-poll", cb, row)
 
-    # VM Console preferences
-    def on_console_accels_changed(self, cb):
-        return self.conf.notify_add("/console/enable-accels", cb)
-    def get_console_accels(self):
-        console_pref = self.conf.get("/console/enable-accels")
-        if console_pref is None:
-            console_pref = False
-        return console_pref
-    def set_console_accels(self, pref):
-        self.conf.set("/console/enable-accels", pref)
-
-    def on_console_scaling_changed(self, cb):
-        return self.conf.notify_add("/console/scaling", cb)
     def get_console_scaling(self):
         return self.conf.get("/console/scaling")
     def set_console_scaling(self, pref):
         self.conf.set("/console/scaling", pref)
 
-    def on_console_resizeguest_changed(self, cb):
-        return self.conf.notify_add("/console/resize-guest", cb)
     def get_console_resizeguest(self):
         val = self.conf.get("/console/resize-guest")
         if val == -1:
@@ -441,120 +483,92 @@ class vmmConfig(object):
     def set_console_resizeguest(self, pref):
         self.conf.set("/console/resize-guest", pref)
 
-    def get_auto_redirection(self):
-        if self.cli_usbredir is not None:
-            return self.cli_usbredir
-        return self.conf.get("/console/auto-redirect")
-    def set_auto_redirection(self, state):
+    def get_auto_usbredir(self):
+        return bool(self.conf.get("/console/auto-redirect"))
+    def set_auto_usbredir(self, state):
         self.conf.set("/console/auto-redirect", state)
 
+    def get_console_autoconnect(self):
+        return bool(self.conf.get("/console/autoconnect"))
+    def set_console_autoconnect(self, val):
+        return self.conf.set("/console/autoconnect", val)
+
     # Show VM details toolbar
     def get_details_show_toolbar(self):
         res = self.conf.get("/details/show-toolbar")
         if res is None:
-            res = True
+            res = True  # pragma: no cover
         return res
     def set_details_show_toolbar(self, state):
         self.conf.set("/details/show-toolbar", state)
 
-    # VM details default size
-    def get_details_window_size(self):
-        w = self.conf.get("/details/window_width")
-        h = self.conf.get("/details/window_height")
-        return (w, h)
-    def set_details_window_size(self, w, h):
-        self.conf.set("/details/window_width", w)
-        self.conf.set("/details/window_height", h)
-
-
     # New VM preferences
-    def get_new_vm_sound(self):
-        return self.conf.get("/new-vm/add-sound")
-    def set_new_vm_sound(self, state):
-        self.conf.set("/new-vm/add-sound", state)
-
     def get_graphics_type(self, raw=False):
         ret = self.conf.get("/new-vm/graphics-type")
         if ret not in ["system", "vnc", "spice"]:
-            ret = "system"
+            ret = "system"  # pragma: no cover
         if ret == "system" and not raw:
             return self.default_graphics_from_config
         return ret
     def set_graphics_type(self, gtype):
         self.conf.set("/new-vm/graphics-type", gtype.lower())
 
-    def get_add_spice_usbredir(self, raw=False):
-        ret = self.conf.get("/new-vm/add-spice-usbredir")
-        if ret not in ["system", "yes", "no"]:
-            ret = "system"
-        if not raw and not self.get_graphics_type() == "spice":
-            return "no"
-        if ret == "system" and not raw:
-            return self.default_add_spice_usbredir
-        return ret
-    def set_add_spice_usbredir(self, val):
-        self.conf.set("/new-vm/add-spice-usbredir", val)
-
     def get_default_storage_format(self, raw=False):
         ret = self.conf.get("/new-vm/storage-format")
         if ret not in ["default", "raw", "qcow2"]:
-            ret = "default"
+            ret = "default"  # pragma: no cover
         if ret == "default" and not raw:
             return self.default_storage_format_from_config
         return ret
     def set_storage_format(self, typ):
         self.conf.set("/new-vm/storage-format", typ.lower())
 
-    def get_default_cpu_setting(self, raw=False, for_cpu=False):
+    def get_default_cpu_setting(self):
         ret = self.conf.get("/new-vm/cpu-default")
-        whitelist = [CPU.SPECIAL_MODE_HOST_MODEL_ONLY,
-                     CPU.SPECIAL_MODE_HOST_MODEL,
-                     CPU.SPECIAL_MODE_HV_DEFAULT]
-
-        if ret not in whitelist:
-            ret = "default"
-        if ret == "default" and not raw:
-            ret = self.cpu_default_from_config
-            if ret not in whitelist:
-                ret = whitelist[0]
-
-        if for_cpu and ret == CPU.SPECIAL_MODE_HOST_MODEL:
-            # host-model has known issues, so use our 'copy cpu'
-            # behavior until host-model does what we need
-            ret = CPU.SPECIAL_MODE_HOST_COPY
 
+        if ret not in DomainCpu.SPECIAL_MODES:
+            ret = DomainCpu.SPECIAL_MODE_APP_DEFAULT  # pragma: no cover
         return ret
     def set_default_cpu_setting(self, val):
         self.conf.set("/new-vm/cpu-default", val.lower())
 
+    def get_default_firmware_setting(self):
+        ret = self.conf.get("/new-vm/firmware")
+        if ret not in ["default", "uefi"]:
+            ret = "default"  # pragma: no cover
+        return ret
+    def set_firmware_setting(self, val):
+        self.conf.set("/new-vm/firmware", val.lower())
+
 
     # URL/Media path history
     def _url_add_helper(self, gsettings_path, url):
         maxlength = 10
-        urls = self.conf.get(gsettings_path)
-        if urls is None:
-            urls = []
+        urls = self.conf.get(gsettings_path) or []
 
         if urls.count(url) == 0 and len(url) > 0 and not url.isspace():
             # The url isn't already in the list, so add it
             urls.insert(0, url)
             if len(urls) > maxlength:
-                del urls[len(urls) - 1]
+                del urls[len(urls) - 1]  # pragma: no cover
             self.conf.set(gsettings_path, urls)
 
+    def add_container_url(self, url):
+        self._url_add_helper("/urls/containers", url)
+    def get_container_urls(self):
+        return self.conf.get("/urls/containers") or []
+
     def add_media_url(self, url):
         self._url_add_helper("/urls/urls", url)
-    def add_kickstart_url(self, url):
-        self._url_add_helper("/urls/kickstarts", url)
+    def get_media_urls(self):
+        return self.conf.get("/urls/urls") or []
+
     def add_iso_path(self, path):
         self._url_add_helper("/urls/isos", path)
-
-    def get_media_urls(self):
-        return self.conf.get("/urls/urls")
-    def get_kickstart_urls(self):
-        return self.conf.get("/urls/kickstarts")
     def get_iso_paths(self):
-        return self.conf.get("/urls/isos")
+        return self.conf.get("/urls/isos") or []
+    def on_iso_paths_changed(self, cb):
+        return self.conf.notify_add("/urls/isos", cb)
 
 
     # Whether to ask about fixing path permissions
@@ -562,7 +576,7 @@ class vmmConfig(object):
         current_list = self.get_perms_fix_ignore() or []
         for path in pathlist:
             if path in current_list:
-                continue
+                continue  # pragma: no cover
             current_list.append(path)
         self.conf.set("/paths/perms-fix-ignore", current_list)
     def get_perms_fix_ignore(self):
@@ -570,24 +584,16 @@ class vmmConfig(object):
 
 
     # Manager view connection list
-    def add_conn(self, uri):
-        if self.test_first_run:
-            return
-
-        uris = self.conf.get("/connections/uris")
-        if uris is None:
-            uris = []
-
-        if uris.count(uri) == 0:
+    def get_conn_uris(self):
+        return self.conf.get("/connections/uris") or []
+    def add_conn_uri(self, uri):
+        uris = self.get_conn_uris()
+        if uri not in uris:
             uris.insert(len(uris) - 1, uri)
             self.conf.set("/connections/uris", uris)
-    def remove_conn(self, uri):
-        uris = self.conf.get("/connections/uris")
-
-        if uris is None:
-            return
-
-        if uris.count(uri) != 0:
+    def remove_conn_uri(self, uri):
+        uris = self.get_conn_uris()
+        if uri in uris:
             uris.remove(uri)
             self.conf.set("/connections/uris", uris)
 
@@ -596,11 +602,6 @@ class vmmConfig(object):
             uris.remove(uri)
             self.conf.set("/connections/autoconnect", uris)
 
-    def get_conn_uris(self):
-        if self.test_first_run:
-            return []
-        return self.conf.get("/connections/uris")
-
     # Manager default window size
     def get_manager_window_size(self):
         w = self.conf.get("/manager-window-width")
@@ -616,12 +617,7 @@ class vmmConfig(object):
         return ((uris is not None) and (uri in uris))
 
     def set_conn_autoconnect(self, uri, val):
-        if self.test_first_run:
-            return
-
-        uris = self.conf.get("/connections/autoconnect")
-        if uris is None:
-            uris = []
+        uris = self.conf.get("/connections/autoconnect") or []
         if not val and uri in uris:
             uris.remove(uri)
         elif val and uri not in uris:
@@ -631,77 +627,23 @@ class vmmConfig(object):
 
 
     # Default directory location dealings
-    def _get_default_dir_key(self, _type):
-        if (_type in [self.CONFIG_DIR_ISO_MEDIA,
-                      self.CONFIG_DIR_FLOPPY_MEDIA]):
-            return "media"
-        if (_type in [self.CONFIG_DIR_IMAGE,
-                      self.CONFIG_DIR_SCREENSHOT]):
-            return _type
-        return None
-
     def get_default_directory(self, conn, _type):
         ignore = conn
-        key = self._get_default_dir_key(_type)
+        browsedata = self.browse_reason_data.get(_type, {})
+        key = browsedata.get("gsettings_key", None)
         path = None
 
         if key:
             path = self.conf.get("/paths/%s-default" % key)
 
-        if not path:
-            if (_type == self.CONFIG_DIR_IMAGE or
-                _type == self.CONFIG_DIR_ISO_MEDIA or
-                _type == self.CONFIG_DIR_FLOPPY_MEDIA):
-                path = os.getcwd()
-
-        logging.debug("directory for type=%s returning=%s", _type, path)
+        log.debug("directory for type=%s returning=%s", _type, path)
         return path
 
     def set_default_directory(self, folder, _type):
-        key = self._get_default_dir_key(_type)
+        browsedata = self.browse_reason_data.get(_type, {})
+        key = browsedata.get("gsettings_key", None)
         if not key:
-            return
+            return  # pragma: no cover
 
-        logging.debug("saving directory for type=%s to %s", key, folder)
+        log.debug("saving directory for type=%s to %s", key, folder)
         self.conf.set("/paths/%s-default" % key, folder)
-
-    # Keyring / VNC password dealings
-    def get_secret_name(self, vm):
-        return "vm-console-" + vm.get_uuid()
-
-    def has_keyring(self):
-        if self.keyring is None:
-            self.keyring = vmmKeyring()
-        return self.keyring.is_available()
-
-    def get_console_password(self, vm):
-        if not self.has_keyring():
-            return ("", "")
-
-        username, keyid = vm.get_console_password()
-
-        if keyid == -1:
-            return ("", "")
-
-        secret = self.keyring.get_secret(keyid)
-        if secret is None or secret.get_name() != self.get_secret_name(vm):
-            return ("", "")
-
-        if (secret.attributes.get("hvuri", None) != vm.conn.get_uri() or
-            secret.attributes.get("uuid", None) != vm.get_uuid()):
-            return ("", "")
-
-        return (secret.get_secret(), username or "")
-
-    def set_console_password(self, vm, password, username=""):
-        if not self.has_keyring():
-            return
-
-        secret = vmmSecret(self.get_secret_name(vm), password,
-                           {"uuid" : vm.get_uuid(),
-                            "hvuri": vm.conn.get_uri()})
-        keyid = self.keyring.add_secret(secret)
-        if keyid is None:
-            return
-
-        vm.set_console_password(username, keyid)
