@@ -129,6 +129,16 @@ class _CapsGuest(XMLBuilder):
                 ret.append(m.canonical)
         return ret
 
+    def is_machine_alias(self, domain, src, tgt):
+        """
+        Determine if machine @src is an alias for machine @tgt
+        """
+        mobjs = (domain and domain.machines) or self.machines
+        for m in mobjs:
+            if m.name == src and m.canonical == tgt:
+                return True
+        return False
+
     def is_kvm_available(self):
         """
         Return True if kvm guests can be installed
@@ -179,6 +189,9 @@ class _CapsInfo(object):
         self.emulator = self.domain.emulator or self.guest.emulator
         self.machines = self.guest.all_machine_names(self.domain)
 
+    def is_machine_alias(self, src, tgt):
+        return self.guest.is_machine_alias(self.domain, src, tgt)
+
 
 class Capabilities(XMLBuilder):
     def __init__(self, *args, **kwargs):
