@@ -8,14 +8,18 @@
 
 import os
 
+from . import cloudinit
 from . import unattended
+from . import volumeupload
 from .installertreemedia import InstallerTreeMedia
 from .installerinject import perform_cdrom_injections
 from ..domain import DomainOs
 from ..devices import DeviceDisk
+from ..guest import Guest
 from ..osdict import OSDB
 from ..logger import log
 from .. import progress
+from .. import xmlutil
 
 
 def _make_testsuite_path(path):
@@ -49,7 +53,7 @@ class Installer(object):
     def __init__(self, conn, cdrom=None, location=None, install_bootdev=None,
             location_kernel=None, location_initrd=None,
             install_kernel=None, install_initrd=None, install_kernel_args=None,
-            no_install=None):
+            no_install=None, is_reinstall=False):
         self.conn = conn
 
         # Entry point for virt-manager 'Customize' wizard to change autostart
@@ -58,11 +62,15 @@ class Installer(object):
         self._install_cdrom_device_added = False
         self._unattended_install_cdrom_device = None
         self._tmpfiles = []
+        self._tmpvols = []
         self._defaults_are_set = False
         self._unattended_data = None
+        self._cloudinit_data = None
 
         self._install_bootdev = install_bootdev
         self._no_install = no_install
+        self._is_reinstall = is_reinstall
+        self._pre_reinstall_xml = None
 
         self._treemedia = None
         self._treemedia_bootconfig = None
@@ -78,10 +86,53 @@ class Installer(object):
                     install_kernel, install_initrd, install_kernel_args)
 
 
+    ##################
+    # Static helpers #
+    ##################
+
+    @staticmethod
+    def cleanup_created_disks(guest, meter):
+        """
+        Remove any disks we created as part of the install. Only ever
+        called by clients.
+        """
+        clean_disks = [d for d in guest.devices.disk if d.storage_was_created]
+
+        for disk in clean_disks:
+            path = disk.get_source_path()
+            log.debug("Removing created disk path=%s vol_object=%s",
+                path, disk.get_vol_object())
+            name = os.path.basename(path)
+
+            try:
+                meter.start(_("Removing disk '%s'") % name, None)
+
+                if disk.get_vol_object():
+                    disk.get_vol_object().delete()
+                else:  # pragma: no cover
+                    # This case technically shouldn't happen here, but
+                    # it's here in case future assumptions change
+                    os.unlink(path)
+
+                meter.end()
+            except Exception as e:  # pragma: no cover
+                log.debug("Failed to remove disk '%s'",
+                    name, exc_info=True)
+                log.error("Failed to remove disk '%s': %s", name, e)
+
+
     ###################
     # Private helpers #
     ###################
 
+    def _make_cdrom_device(self, path):
+        dev = DeviceDisk(self.conn)
+        dev.device = dev.DEVICE_CDROM
+        dev.set_source_path(path)
+        dev.sync_path_props()
+        dev.validate()
+        return dev
+
     def _cdrom_path(self):
         if self._treemedia:
             return self._treemedia.cdrom_path()
@@ -93,13 +144,19 @@ class Installer(object):
         if not bool(self._cdrom_path()):
             return
 
-        dev = DeviceDisk(self.conn)
-        dev.device = dev.DEVICE_CDROM
-        dev.path = self._cdrom_path()
-        dev.sync_path_props()
-        dev.validate()
         self._install_cdrom_device_added = True
 
+        if self._is_reinstall:
+            cdroms = [d for d in guest.devices.disk if d.is_cdrom()]
+            if cdroms:
+                dev = cdroms[0]
+                dev.set_source_path(self._cdrom_path())
+                return
+
+        dev = self._make_cdrom_device(self._cdrom_path())
+        if self._is_reinstall:
+            dev.set_defaults(guest)
+
         # Insert the CDROM before any other CDROM, so boot=cdrom picks
         # it as the priority
         for idx, disk in enumerate(guest.devices.disk):
@@ -115,36 +172,34 @@ class Installer(object):
             # Keep media attached for windows which has a multi stage install
             return
         for disk in guest.devices.disk:
-            if disk.is_cdrom() and disk.path == self._cdrom_path():
-                disk.path = None
+            if (disk.is_cdrom() and
+                disk.get_source_path() == self._cdrom_path()):
+                disk.set_source_path(None)
                 disk.sync_path_props()
                 break
 
     def _add_unattended_install_cdrom_device(self, guest, location):
         if self._unattended_install_cdrom_device:
             return  # pragma: no cover
-        dev = DeviceDisk(self.conn)
-        dev.device = dev.DEVICE_CDROM
-        dev.path = location
-        dev.sync_path_props()
-        dev.validate()
+
+        dev = self._make_cdrom_device(location)
         dev.set_defaults(guest)
-        self._unattended_install_cdrom_device = dev
-        guest.add_device(self._unattended_install_cdrom_device)
+        self._unattended_install_cdrom_device = dev.target
+        guest.add_device(dev)
 
         if self.conn.in_testsuite():
-            # pylint: disable=protected-access
             # Hack to set just the XML path differently for the test suite.
             # Setting this via regular 'path' will error that it doesn't exist
-            dev._source_file = _make_testsuite_path(location)
+            dev.source.file = _make_testsuite_path(location)
 
     def _remove_unattended_install_cdrom_device(self, guest):
-        dummy = guest
         if not self._unattended_install_cdrom_device:
             return
 
-        self._unattended_install_cdrom_device.path = None
-        self._unattended_install_cdrom_device.sync_path_props()
+        disk = [d for d in guest.devices.disk if
+                d.target == self._unattended_install_cdrom_device][0]
+        disk.set_source_path(None)
+        disk.sync_path_props()
 
     def _build_boot_order(self, guest, bootdev):
         bootorder = [bootdev]
@@ -210,39 +265,89 @@ class Installer(object):
         ram = guest.osinfo.get_network_install_required_ram(guest)
         ram = (ram or 0) // 1024
         if ram > guest.currentMemory:
-            log.warning(_("Overriding memory to %s MiB needed for %s "
-                "network install."), ram // 1024, guest.osinfo.name)
+            msg = (_("Overriding memory to %(number)s MiB needed for "
+                "%(osname)s network install.") %
+                {"number": ram // 1024, "osname": guest.osinfo.name})
+            log.warning(msg)
             guest.currentMemory = ram
 
 
-    ##########################
-    # Internal API overrides #
-    ##########################
+    ################
+    # Internal API #
+    ################
 
-    def _prepare_unattended_data(self, guest, script):
-        expected_filename = script.get_expected_filename()
-        unattended_cmdline = script.generate_cmdline()
-        log.debug("Generated unattended cmdline: %s", unattended_cmdline)
+    def _should_upload_media(self, guest):
+        """
+        Return True if we should upload media to the connection scratchdir.
+        This doesn't consider if there is any media to upload, just whether
+        we _should_ upload if there _is_ media.
+        """
+        scratchdir = InstallerTreeMedia.make_scratchdir(guest)
+        system_scratchdir = InstallerTreeMedia.get_system_scratchdir(guest)
 
-        scriptpath = script.write()
-        self._tmpfiles.append(scriptpath)
+        if self.conn.is_remote():
+            return True
+        if self.conn.is_unprivileged():
+            return False
+        if scratchdir == system_scratchdir:
+            return False  # pragma: no cover
+        return True
 
-        iso = perform_cdrom_injections([(scriptpath, expected_filename)],
-                InstallerTreeMedia.make_scratchdir(guest))
+    def _upload_media(self, guest, meter, paths):
+        system_scratchdir = InstallerTreeMedia.get_system_scratchdir(guest)
+
+        if (not self._should_upload_media(guest) and
+            not xmlutil.in_testsuite()):
+            # We have access to system scratchdir, don't jump through hoops
+            log.debug("Have access to preferred scratchdir so"
+                        " nothing to upload")  # pragma: no cover
+            return paths  # pragma: no cover
+
+        if not guest.conn.support_remote_url_install():
+            # Needed for the test_urls suite
+            log.debug("Media upload not supported")  # pragma: no cover
+            return paths  # pragma: no cover
+
+        newpaths, tmpvols = volumeupload.upload_paths(
+                guest.conn, system_scratchdir, meter, paths)
+        self._tmpvols += tmpvols
+        return newpaths
+
+    def _prepare_unattended_data(self, guest, meter, scripts):
+        scratchdir = InstallerTreeMedia.make_scratchdir(guest)
+
+        injections = []
+        for script in scripts:
+            expected_filename = script.get_expected_filename()
+            unattended_cmdline = script.generate_cmdline()
+            log.debug("Generated unattended cmdline: %s", unattended_cmdline)
+
+            scriptpath = script.write()
+            self._tmpfiles.append(scriptpath)
+            injections.append((scriptpath, expected_filename))
+
+        drivers_location = guest.osinfo.get_pre_installable_drivers_location(
+                guest.os.arch)
+        drivers = unattended.download_drivers(
+                drivers_location, scratchdir, meter)
+        injections.extend(drivers)
+        self._tmpfiles.extend([driverpair[0] for driverpair in drivers])
+
+        iso = perform_cdrom_injections(injections, scratchdir)
         self._tmpfiles.append(iso)
+        iso = self._upload_media(guest, meter, [iso])[0]
         self._add_unattended_install_cdrom_device(guest, iso)
 
-    def _prepare_unattended_script(self, guest, meter):
+    def _prepare_unattended_scripts(self, guest, meter):
         url = None
+        os_tree = None
         if self._treemedia:
             if self._treemedia.is_network_url():
                 url = self.location
             os_media = self._treemedia.get_os_media(guest, meter)
+            os_tree = self._treemedia.get_os_tree(guest, meter)
             injection_method = "initrd"
         else:
-            if self.conn.is_remote():
-                raise RuntimeError("Unattended method=cdrom installs are "
-                        "not yet supported for remote connections.")
             if not guest.osinfo.is_windows():
                 log.warning("Attempting unattended method=cdrom injection "
                         "for a non-windows OS. If this doesn't work, try "
@@ -251,28 +356,59 @@ class Installer(object):
             os_media = osguess[1] if osguess else None
             injection_method = "cdrom"
 
-        return unattended.prepare_install_script(
-                guest, self._unattended_data, url, os_media, injection_method)
+        return unattended.prepare_install_scripts(
+                guest, self._unattended_data, url,
+                os_media, os_tree, injection_method)
+
+    def _prepare_treemedia(self, guest, meter, unattended_scripts):
+        kernel, initrd, kernel_args = self._treemedia.prepare(guest, meter,
+                unattended_scripts)
+
+        paths = [kernel, initrd]
+        kernel, initrd = self._upload_media(guest, meter, paths)
+        self._treemedia_bootconfig = (kernel, initrd, kernel_args)
+
+    def _prepare_cloudinit(self, guest, meter):
+        scratchdir = InstallerTreeMedia.make_scratchdir(guest)
+        filepairs = cloudinit.create_files(scratchdir, self._cloudinit_data)
+        for filepair in filepairs:
+            self._tmpfiles.append(filepair[0])
+
+        iso = perform_cdrom_injections(filepairs, scratchdir, cloudinit=True)
+        self._tmpfiles.append(iso)
+        iso = self._upload_media(guest, meter, [iso])[0]
+        self._add_unattended_install_cdrom_device(guest, iso)
 
     def _prepare(self, guest, meter):
-        unattended_script = None
+        if self._is_reinstall:
+            self._pre_reinstall_xml = guest.get_xml()
+
+        unattended_scripts = None
         if self._unattended_data:
-            unattended_script = self._prepare_unattended_script(guest, meter)
+            unattended_scripts = self._prepare_unattended_scripts(guest, meter)
 
         if self._treemedia:
-            self._treemedia_bootconfig = self._treemedia.prepare(guest, meter,
-                    unattended_script)
+            self._prepare_treemedia(guest, meter, unattended_scripts)
 
-        elif unattended_script:
-            self._prepare_unattended_data(guest, unattended_script)
+        elif unattended_scripts:
+            self._prepare_unattended_data(guest, meter, unattended_scripts)
+
+        elif self._cloudinit_data:
+            self._prepare_cloudinit(guest, meter)
 
     def _cleanup(self, guest):
         if self._treemedia:
             self._treemedia.cleanup(guest)
 
+        for vol in self._tmpvols:
+            log.debug("Removing volume '%s'", vol.name())
+            vol.delete(0)
+        self._tmpvols = []
+
         for f in self._tmpfiles:
             log.debug("Removing %s", str(f))
             os.unlink(f)
+        self._tmpfiles = []
 
     def _get_postinstall_bootdev(self, guest):
         if self.cdrom and self._no_install:
@@ -330,6 +466,8 @@ class Installer(object):
         """
         if self._defaults_are_set:
             return
+        if self._is_reinstall:
+            self._pre_reinstall_xml = guest.get_xml()
 
         self._add_install_cdrom_device(guest)
 
@@ -337,7 +475,8 @@ class Installer(object):
             bootdev = self._get_postinstall_bootdev(guest)
             guest.os.bootorder = self._build_boot_order(guest, bootdev)
 
-        guest.set_defaults(None)
+        if not self._is_reinstall:
+            guest.set_defaults(None)
         self._defaults_are_set = True
 
     def get_search_paths(self, guest):
@@ -346,7 +485,10 @@ class Installer(object):
         for to perform this install.
         """
         search_paths = []
-        if self._treemedia:
+        if ((self._treemedia or
+             self._cloudinit_data or
+             self._unattended_data) and
+             not self._should_upload_media(guest)):
             search_paths.append(InstallerTreeMedia.make_scratchdir(guest))
         if self._cdrom_path():
             search_paths.append(self._cdrom_path())
@@ -364,10 +506,14 @@ class Installer(object):
                     self._install_bootdev or
                     self._treemedia)
 
+    def _requires_postboot_xml_changes(self):
+        if self.has_cloudinit() or self.has_unattended():
+            return True
+        return self.has_install_phase()
+
     def options_specified(self):
         """
         Return True if some explicit install option was actually passed in
-        Validate that some install option was actually passed in.
         """
         if self._no_install:
             return True
@@ -401,55 +547,55 @@ class Installer(object):
     def set_unattended_data(self, unattended_data):
         self._unattended_data = unattended_data
 
+    def set_cloudinit_data(self, cloudinit_data):
+        self._cloudinit_data = cloudinit_data
+
+    def has_cloudinit(self):
+        return bool(self._cloudinit_data)
+    def has_unattended(self):
+        return bool(self._unattended_data)
+
+    def get_generated_password(self):
+        if self._cloudinit_data:
+            return self._cloudinit_data.get_password_if_generated()
+
 
     ##########################
     # guest install handling #
     ##########################
 
-    def _prepare_get_install_xml(self, guest):
-        # We do a shallow copy of the OS block here, so that we can
-        # set the install time properties but not permanently overwrite
-        # any config the user explicitly requested.
-        data = (guest.os.bootorder, guest.os.kernel, guest.os.initrd,
-                guest.os.kernel_args, guest.on_reboot, guest.currentMemory,
-                guest.memory)
-        return data
+    def _build_postboot_xml(self, final_xml, meter):
+        initial_guest = Guest(self.conn, parsexml=final_xml)
+        self._alter_bootconfig(initial_guest)
+        self._alter_install_resources(initial_guest, meter)
+        if self.has_cloudinit():
+            initial_guest.set_smbios_serial_cloudinit()
 
-    def _finish_get_install_xml(self, guest, data):
-        (guest.os.bootorder, guest.os.kernel, guest.os.initrd,
-                guest.os.kernel_args, guest.on_reboot, guest.currentMemory,
-                guest.memory) = data
+        final_guest = Guest(self.conn, parsexml=final_xml)
+        self._remove_install_cdrom_media(final_guest)
+        self._remove_unattended_install_cdrom_device(final_guest)
 
-    def _get_install_xml(self, guest, meter):
-        data = self._prepare_get_install_xml(guest)
-        try:
-            self._alter_bootconfig(guest)
-            self._alter_install_resources(guest, meter)
-            ret = guest.get_xml()
-            return ret
-        finally:
-            self._remove_install_cdrom_media(guest)
-            self._remove_unattended_install_cdrom_device(guest)
-            self._finish_get_install_xml(guest, data)
+        return initial_guest.get_xml(), final_guest.get_xml()
 
     def _build_xml(self, guest, meter):
-        install_xml = None
-        if self.has_install_phase():
-            install_xml = self._get_install_xml(guest, meter)
+        initial_xml = None
         final_xml = guest.get_xml()
+        if self._requires_postboot_xml_changes():
+            initial_xml, final_xml = self._build_postboot_xml(final_xml, meter)
+        final_xml = self._pre_reinstall_xml or final_xml
 
-        log.debug("Generated install XML: %s",
-            (install_xml and ("\n" + install_xml) or "None required"))
-        log.debug("Generated boot XML: \n%s", final_xml)
+        log.debug("Generated initial_xml: %s",
+            (initial_xml and ("\n" + initial_xml) or "None required"))
+        log.debug("Generated final_xml: \n%s", final_xml)
 
-        return install_xml, final_xml
+        return initial_xml, final_xml
 
-    def _manual_transient_create(self, install_xml, final_xml, needs_boot):
+    def _manual_transient_create(self, initial_xml, final_xml, needs_boot):
         """
         For hypervisors (like vz) that don't implement createXML,
         we need to define+start, and undefine on start failure
         """
-        domain = self.conn.defineXML(install_xml or final_xml)
+        domain = self.conn.defineXML(initial_xml or final_xml)
         if not needs_boot:
             return domain
 
@@ -463,12 +609,12 @@ class Installer(object):
                 pass
             raise
 
-        if install_xml and install_xml != final_xml:
+        if initial_xml and initial_xml != final_xml:
             domain = self.conn.defineXML(final_xml)
         return domain
 
     def _create_guest(self, guest,
-                      meter, install_xml, final_xml, doboot, transient):
+                      meter, initial_xml, final_xml, doboot, transient):
         """
         Actually do the XML logging, guest defining/creating
 
@@ -476,19 +622,19 @@ class Installer(object):
         """
         meter_label = _("Creating domain...")
         meter = progress.ensure_meter(meter)
-        meter.start(size=None, text=meter_label)
-        needs_boot = doboot or self.has_install_phase()
+        meter.start(meter_label, None)
+        needs_boot = doboot or bool(initial_xml)
 
-        if guest.type == "vz":
+        if guest.type == "vz" and not self._is_reinstall:
             if transient:
                 raise RuntimeError(_("Domain type 'vz' doesn't support "
                     "transient installs."))
             domain = self._manual_transient_create(
-                    install_xml, final_xml, needs_boot)
+                    initial_xml, final_xml, needs_boot)
 
         else:
             if transient or needs_boot:
-                domain = self.conn.createXML(install_xml or final_xml, 0)
+                domain = self.conn.createXML(initial_xml or final_xml, 0)
             if not transient:
                 domain = self.conn.defineXML(final_xml)
 
@@ -497,6 +643,7 @@ class Installer(object):
                           domain.XMLDesc(0))
         except Exception as e:  # pragma: no cover
             log.debug("Error fetching XML from libvirt object: %s", e)
+        meter.end()
         return domain
 
     def _flag_autostart(self, domain):
@@ -516,7 +663,7 @@ class Installer(object):
     # Public install API #
     ######################
 
-    def start_install(self, guest, meter=None,
+    def start_install(self, user_guest, meter=None,
                       dry=False, return_xml=False,
                       doboot=True, transient=False):
         """
@@ -525,22 +672,28 @@ class Installer(object):
 
         :param return_xml: Don't create the guest, just return generated XML
         """
-        guest.validate_name(guest.conn, guest.name)
-        self.set_install_defaults(guest)
+        if not self._is_reinstall and not return_xml:
+            Guest.validate_name(self.conn, user_guest.name)
+        self.set_install_defaults(user_guest)
+        disks = user_guest.devices.disk[:]
+
+        # All installer XML alterations are made on this guest instance,
+        # so the user_guest instance is left intact
+        guest = Guest(self.conn, parsexml=user_guest.get_xml())
 
         try:
             self._prepare(guest, meter)
 
-            if not dry:
-                for dev in guest.devices.disk:
+            if not dry and not self._is_reinstall:
+                for dev in disks:
                     dev.build_storage(meter)
 
-            install_xml, final_xml = self._build_xml(guest, meter)
+            initial_xml, final_xml = self._build_xml(guest, meter)
             if dry or return_xml:
-                return (install_xml, final_xml)
+                return (initial_xml, final_xml)
 
             domain = self._create_guest(
-                    guest, meter, install_xml, final_xml,
+                    guest, meter, initial_xml, final_xml,
                     doboot, transient)
 
             if self.autostart:
@@ -548,36 +701,3 @@ class Installer(object):
             return domain
         finally:
             self._cleanup(guest)
-
-    def get_created_disks(self, guest):
-        return [d for d in guest.devices.disk if d.storage_was_created]
-
-    def cleanup_created_disks(self, guest, meter):
-        """
-        Remove any disks we created as part of the install. Only ever
-        called by clients.
-        """
-        clean_disks = self.get_created_disks(guest)
-        if not clean_disks:
-            return
-
-        for disk in clean_disks:
-            log.debug("Removing created disk path=%s vol_object=%s",
-                disk.path, disk.get_vol_object())
-            name = os.path.basename(disk.path)
-
-            try:
-                meter.start(size=None, text=_("Removing disk '%s'") % name)
-
-                if disk.get_vol_object():
-                    disk.get_vol_object().delete()
-                else:  # pragma: no cover
-                    # This case technically shouldn't happen here, but
-                    # it's here incase future assumptions change
-                    os.unlink(disk.path)
-
-                meter.end(0)
-            except Exception as e:  # pragma: no cover
-                log.debug("Failed to remove disk '%s'",
-                    name, exc_info=True)
-                log.error("Failed to remove disk '%s': %s", name, e)
