@@ -156,7 +156,10 @@ def action_edit(guest, options, parserclass):
     if options.os_variant is not None:
         fail(_("--os-variant/--osinfo is not supported with --edit"))
 
-    return cli.parse_option_strings(options, guest, inst, editing=True)
+    devs = []
+    for editinst in xmlutil.listify(inst):
+        devs += cli.run_parser(options, guest, parserclass, editinst=editinst)
+    return devs
 
 
 def action_add_device(guest, options, parserclass, devs):
@@ -168,8 +171,7 @@ def action_add_device(guest, options, parserclass, devs):
         for dev in devs:
             guest.add_device(dev)
     else:
-        devs = cli.parse_option_strings(options, guest, None)
-        devs = xmlutil.listify(devs)
+        devs = cli.run_parser(options, guest, parserclass)
         for dev in devs:
             dev.set_defaults(guest)
 
@@ -199,21 +201,14 @@ def action_remove_device(guest, options, parserclass):
     return devs
 
 
-def action_build_xml(conn, options, parserclass, guest):
+def action_build_xml(options, parserclass, guest):
     if not parserclass.guest_propname:
         fail(_("--build-xml not supported for --%s") %
              parserclass.cli_arg_name)
     if options.os_variant is not None:
         fail(_("--os-variant/--osinfo is not supported with --build-xml"))
 
-    inst = parserclass.lookup_prop(guest)
-    if parserclass.prop_is_list(guest):
-        inst = inst.new()
-    else:
-        inst = inst.__class__(conn)
-
-    devs = cli.parse_option_strings(options, guest, inst)
-    devs = xmlutil.listify(devs)
+    devs = cli.run_parser(options, guest, parserclass)
     for dev in devs:
         dev.set_defaults(guest)
     return devs
@@ -507,10 +502,10 @@ def main(conn=None):
     vm_is_running = bool(active_xmlobj)
 
     if options.build_xml:
-        devs = action_build_xml(conn, options, parserclass, inactive_xmlobj)
+        devs = action_build_xml(options, parserclass, inactive_xmlobj)
         for dev in devs:
             # pylint: disable=no-member
-            print_stdout(dev.get_xml())
+            print_stdout(xmlutil.unindent_device_xml(dev.get_xml()))
         return 0
 
     devs = None
